#ifndef __CLOUD_H__
#define __CLOUD_H__
/* Copyright (c) 2009 PrimeBase Technologies GmbH, Germany
 *
 * PrimeBase Media Stream for MySQL
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  Created by Barry Leslie on 3/20/09.
 *
 */
#include "CSMd5.h"

#define MAX_NODE_SIZE	64	 
#define MAX_PUBLIC_KEY	21	 
#define MAX_PRIVATE_KEY	41

typedef struct {
	char node[MAX_NODE_SIZE];
	char public_key[MAX_PUBLIC_KEY];	
	char private_key[MAX_PRIVATE_KEY];
} CloudKey;

/* NOTES:
 *
 * - TODO: If cl_deleteData() fails then the BLOB deletion must fail and be rescheduled to try again
 *			later.
 * - TODO: The BLOB transfer must be done in the background after the BLOB is referenced. A local copy
 *			should be kept of the BLOB until we are confident the cload version is available. At this
 *			point the local copy should be deleted. To impliment this something similar to the temp logs 
 *			may be required.
 * - TODO: Copying of BLOBs from one database to another needs to be handled. Look for copyBlob() and 
 *			resetBlobHead(). There are 3 cases to handle bepending on if the databases involved use
 *			cload storage.
 */
class CloudDB: public CSRefObject {
private:
	csWord4 sleep_time; // the time to sleep between restartable events in milliseconds.
	time_t	keep_alive; // The length of time a redirect URL will remain valid. In seconds.

	CloudKey			keys;
	
	void cl_setObjectKey(const char *key) 
	{
		clObjectKey->setLength(clObjectKeyBaseLength); 
		clObjectKey->append(key); 
	}
	void cl_clearObjectKey() {clObjectKey->setLength(clObjectKeyBaseLength);}
	
public:
	CSStringBuffer		*clObjectKey;
	csWord4				clObjectKeyBaseLength;
	
	csWord4 max_retries;
	csWord4 status;
	bool can_retry;
	bool throw_error;
	bool tracing;
	bool file_not_found;

	CloudDB(CloudDB *cloud);
	CloudDB(CSFile *cloud_info_file, csWord4 db_id, time_t keep_alive = (5 * 60));
	~CloudDB();
	
	CloudDB *cl_clone();
	void cl_create_db();
	void cl_drop_db();
	bool cl_db_exists();
	
	void cl_putData(const char *key, CSFile *inFile, off_t f_offset, off_t size, Md5Digest *md5);
	off_t cl_getData(const char *key, char *data, off_t size);
	CSString *cl_getDataURL(const char *key);
	void cl_deleteData(const char *key);
	
private:
	void clearState() {file_not_found = can_retry = throw_error = false; status = 0;}
};

#endif // __CLOUD_H__
